@@ -3,7 +3,9 @@
 
 """Contains the PyTorch Lightning wrapper modules for CIFAR10 and CIFAR100 dataset."""
 
-from typing import List, Optional, Type, Literal, Union, Dict, Any, Tuple
+import enum
+from typing import List, Optional, Type, Union, Dict, Any, Tuple
+from torchfl.federated.fl_params import FLParams
 from torchfl.models.core.cifar.cifar10.alexnet import AlexNet as CIFAR10AlexNet
 from torchfl.models.core.cifar.cifar10.densenet import (
     DenseNet121 as CIFAR10DenseNet121,
@@ -94,7 +96,7 @@ from torchfl.models.core.cifar.cifar100.vgg import (
 )
 import pytorch_lightning as pl
 import torch.nn as nn
-from torchfl.compatibility import OPTIMIZERS_LITERAL, OPTIMIZERS_BY_NAME
+from torchfl.compatibility import OPTIMIZERS_TYPE, OPTIMIZERS_BY_NAME
 from torch import Tensor, optim
 
 pl.seed_everything(42)
@@ -138,75 +140,43 @@ CIFAR_MODELS: List[str] = [
     "vgg19_bn",
 ]
 
-CIFAR_MODELS_LITERAL: Type[
-    Literal[
-        "alexnet",
-        "densenet121",
-        "densenet161",
-        "densenet169",
-        "densenet201",
-        "lenet",
-        "mobilenetv2",
-        "mobilenetv3small",
-        "mobilenetv3large",
-        "resnet18",
-        "resnet34",
-        "resnet50",
-        "resnet101",
-        "resnet152",
-        "resnext50_32x4d",
-        "resnext101_32x8d",
-        "wideresnet50_2",
-        "wideresnet101_2",
-        "shufflenetv2_x0_5",
-        "shufflenetv2_x1_0",
-        "shufflenetv2_x1_5",
-        "shufflenetv2_x2_0",
-        "squeezenet1_0",
-        "squeezenet1_1",
-        "vgg11",
-        "vgg11_bn",
-        "vgg13",
-        "vgg13_bn",
-        "vgg16",
-        "vgg16_bn",
-        "vgg19",
-        "vgg19_bn",
-    ]
-] = Literal[  # type: ignore
-    "alexnet",
-    "densenet121",
-    "densenet161",
-    "densenet169",
-    "densenet201",
-    "lenet",
-    "mobilenetv2",
-    "mobilenetv2small",
-    "mobilenetv3large",
-    "resnet18",
-    "resnet34",
-    "resnet50",
-    "resnet101",
-    "resnet152",
-    "resnext50_32x4d",
-    "resnext101_32x8d",
-    "wideresnet50_2",
-    "wideresnet101_2",
-    "shufflenetv2_x0_5",
-    "shufflenetv2_x1_0",
-    "shufflenetv2_x1_5",
-    "shufflenetv2_x2_0",
-    "squeezenet1_0",
-    "squeezenet1_1",
-    "vgg11",
-    "vgg11_bn",
-    "vgg13",
-    "vgg13_bn",
-    "vgg16",
-    "vgg16_bn",
-    "vgg19",
-    "vgg19_bn",
-]
+
+class CIFAR_MODELS_ENUM(enum.Enum):
+    """Enum for supported CIFAR models."""
+
+    ALEXNET = "alexnet"
+    DENSENET121 = "densenet121"
+    DENSENET161 = "densenet161"
+    DENSENET169 = "densenet169"
+    DENSENET201 = "densenet201"
+    LENET = "lenet"
+    MOBILENETV2 = "mobilenetv2"
+    MOBILENETV3SMALL = "mobilenetv3small"
+    MOBILENETV3LARGE = "mobilenetv3large"
+    RESNET18 = "resnet18"
+    RESNET34 = "resnet34"
+    RESNET50 = "resnet50"
+    RESNET101 = "resnet101"
+    RESNET152 = "resnet152"
+    RESNEXT50_32X4D = "resnext50_32x4d"
+    RESNEXT101_32X8D = "resnext101_32x8d"
+    WIDERESNET50_2 = "wideresnet50_2"
+    WIDERESNET101_2 = "wideresnet101_2"
+    SHUFFLENETV2_X0_5 = "shufflenetv2_x0_5"
+    SHUFFLENETV2_X1_0 = "shufflenetv2_x1_0"
+    SHUFFLENETV2_X1_5 = "shufflenetv2_x1_5"
+    SHUFFLENETV2_X2_0 = "shufflenetv2_x2_0"
+    SQUEEZENET1_0 = "squeezenet1_0"
+    SQUEEZENET1_1 = "squeezenet1_1"
+    VGG11 = "vgg11"
+    VGG11_BN = "vgg11_bn"
+    VGG13 = "vgg13"
+    VGG13_BN = "vgg13_bn"
+    VGG16 = "vgg16"
+    VGG16_BN = "vgg16_bn"
+    VGG19 = "vgg19"
+    VGG19_BN = "vgg19_bn"
+
 
 CIFAR10_MODEL_TYPE = Union[
     Type[CIFAR10AlexNet],
@@ -401,30 +371,38 @@ class CIFAR10(pl.LightningModule):
 
     def __init__(
         self,
-        model_name: CIFAR_MODELS_LITERAL,  # type: ignore
-        optimizer_name: OPTIMIZERS_LITERAL,  # type: ignore
+        model_name: CIFAR_MODELS_ENUM,
+        optimizer_name: OPTIMIZERS_TYPE,
         optimizer_hparams: Dict[str, Any],
         model_hparams: Optional[Dict[str, Any]] = None,
+        fl_hparams: Optional[FLParams] = None,
     ) -> None:
         """Default constructor.
 
         Args:
-            - model_name (CIFAR_MODELS_LITERAL): Name of the model to be used. Only choose from the available models.
-            - optimizer_name (OPTIMIZERS_LITERAL): Name of optimizer to be used. Only choose from the available models.
+            - model_name (str): Name of the model to be used. Only choose from the available models.
+            - optimizer_name (str): Name of optimizer to be used. Only choose from the available models.
             - optimizer_hparams(Dict[str, Any]): Hyperparameters to initialize the optimizer.
             - model_hparams (Optional[Dict[str, Any]], optional): Optional override the default model hparams. Defaults to None.
+            - fl_hparams (Optional[FLParams], optional): Optional override the default federated learning hparams. Defaults to None.
         """
         super().__init__()
         self.model = create_model(
-            dataset_name="cifar10", model_name=model_name, model_hparams=model_hparams
+            dataset_name="cifar10",
+            model_name=model_name.value,
+            model_hparams=model_hparams,
+        )
+        self.fl_hparams: Optional[Dict[str, Any]] = (
+            fl_hparams.as_dict() if fl_hparams else None
         )
         combined_hparams: Dict[str, Any] = {
             "model_hparams": vars(self.model.hparams),
             "optimizer_hparams": {
                 "optimizer_name": optimizer_name,
-                "optimizer_fn": OPTIMIZERS_BY_NAME[optimizer_name],
+                "optimizer_fn": OPTIMIZERS_BY_NAME[optimizer_name.value],
                 "config": optimizer_hparams,
             },
+            "fl_hparams": vars(fl_hparams.as_simple_namespace()) if fl_hparams else {},
         }
         self.save_hyperparameters(combined_hparams)
         self.loss_module = nn.CrossEntropyLoss()
@@ -468,10 +446,25 @@ class CIFAR10(pl.LightningModule):
         loss: Tensor = self.loss_module(preds, labels)
         acc: Tensor = (preds.argmax(dim=-1) == labels).float().mean()
 
-        # Logs the accuracy per epoch (weighted average over batches)
-        self.log("batch_idx", batch_idx)
-        self.log("train_acc", acc, on_step=False, on_epoch=True)
-        self.log("train_loss", loss)
+        # Logs the fl-related metrics if fl_hparams is not None
+        if self.fl_hparams is not None:
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_train_loss",
+                loss,
+                on_step=False,
+                on_epoch=True,
+            )
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_train_acc",
+                acc,
+                on_step=False,
+                on_epoch=True,
+            )
+        else:
+            # Logs the accuracy per epoch (weighted average over batches)
+            self.log("batch_idx", batch_idx)
+            self.log("train_acc", acc, on_step=False, on_epoch=True)
+            self.log("train_loss", loss, on_step=False, on_epoch=True)
         return loss
 
     def validation_step(  # type: ignore
@@ -484,11 +477,30 @@ class CIFAR10(pl.LightningModule):
             - batch_idx (int): Index of the batch.
         """
         imgs, labels = batch
-        preds: Tensor = self.model(imgs).argmax(dim=-1)
+        preds: Tensor = self.model(imgs)
+        loss: Tensor = self.loss_module(preds, labels)
+        preds = self.model(imgs).argmax(dim=-1)
         acc: Tensor = (labels == preds).float().mean()
-        # By default logs it per epoch (weighted average over batches)
-        self.log("batch_idx", batch_idx)
-        self.log("val_acc", acc)
+
+        # Logs the fl-related metrics if fl_hparams is not None
+        if self.fl_hparams is not None:
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_loss",
+                loss,
+                on_step=False,
+                on_epoch=True,
+            )
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_acc",
+                acc,
+                on_step=False,
+                on_epoch=True,
+            )
+        else:
+            # By default logs it per epoch (weighted average over batches), and returns it afterwards
+            self.log("batch_idx", batch_idx)
+            self.log("test_loss", loss, on_step=False, on_epoch=True)
+            self.log("test_acc", acc, on_step=False, on_epoch=True)
 
     def test_step(  # type: ignore
         self, batch: Tuple[Tensor, Tensor], batch_idx: int
@@ -500,11 +512,30 @@ class CIFAR10(pl.LightningModule):
             - batch_idx (int): Index of the batch.
         """
         imgs, labels = batch
-        preds: Tensor = self.model(imgs).argmax(dim=-1)
+        preds: Tensor = self.model(imgs)
+        loss: Tensor = self.loss_module(preds, labels)
+        preds = self.model(imgs).argmax(dim=-1)
         acc: Tensor = (labels == preds).float().mean()
-        # By default logs it per epoch (weighted average over batches), and returns it afterwards
-        self.log("batch_idx", batch_idx)
-        self.log("test_acc", acc)
+
+        # Logs the fl-related metrics if fl_hparams is not None
+        if self.fl_hparams is not None:
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_loss",
+                loss,
+                on_step=False,
+                on_epoch=True,
+            )
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_acc",
+                acc,
+                on_step=False,
+                on_epoch=True,
+            )
+        else:
+            # By default logs it per epoch (weighted average over batches), and returns it afterwards
+            self.log("batch_idx", batch_idx)
+            self.log("test_loss", loss, on_step=False, on_epoch=True)
+            self.log("test_acc", acc, on_step=False, on_epoch=True)
 
 
 class CIFAR100(pl.LightningModule):
@@ -512,30 +543,38 @@ class CIFAR100(pl.LightningModule):
 
     def __init__(
         self,
-        model_name: CIFAR_MODELS_LITERAL,  # type: ignore
-        optimizer_name: OPTIMIZERS_LITERAL,  # type: ignore
+        model_name: CIFAR_MODELS_ENUM,
+        optimizer_name: OPTIMIZERS_TYPE,
         optimizer_hparams: Dict[str, Any],
         model_hparams: Optional[Dict[str, Any]] = None,
+        fl_hparams: Optional[FLParams] = None,
     ) -> None:
         """Default constructor.
 
         Args:
-            - model_name (CIFAR_MODELS_LITERAL): Name of the model to be used. Only choose from the available models.
-            - optimizer_name (OPTIMIZERS_LITERAL): Name of optimizer to be used. Only choose from the available models.
+            - model_name (str): Name of the model to be used. Only choose from the available models.
+            - optimizer_name (str): Name of optimizer to be used. Only choose from the available models.
             - optimizer_hparams(Dict[str, Any]): Hyperparameters to initialize the optimizer.
             - model_hparams (Optional[Dict[str, Any]], optional): Optional override the default model hparams. Defaults to None.
+            - fl_hparams (Optional[FLParams], optional): Optional override the default federated learning hparams. Defaults to None.
         """
         super().__init__()
         self.model = create_model(
-            dataset_name="cifar100", model_name=model_name, model_hparams=model_hparams
+            dataset_name="cifar100",
+            model_name=model_name.value,
+            model_hparams=model_hparams,
+        )
+        self.fl_hparams: Optional[Dict[str, Any]] = (
+            fl_hparams.as_dict() if fl_hparams else None
         )
         combined_hparams: Dict[str, Any] = {
             "model_hparams": vars(self.model.hparams),
             "optimizer_hparams": {
                 "optimizer_name": optimizer_name,
-                "optimizer_fn": OPTIMIZERS_BY_NAME[optimizer_name],
+                "optimizer_fn": OPTIMIZERS_BY_NAME[optimizer_name.value],
                 "config": optimizer_hparams,
             },
+            "fl_hparams": vars(fl_hparams.as_simple_namespace()) if fl_hparams else {},
         }
         self.save_hyperparameters(combined_hparams)
         self.loss_module = nn.CrossEntropyLoss()
@@ -579,10 +618,25 @@ class CIFAR100(pl.LightningModule):
         loss: Tensor = self.loss_module(preds, labels)
         acc: Tensor = (preds.argmax(dim=-1) == labels).float().mean()
 
-        # Logs the accuracy per epoch (weighted average over batches)
-        self.log("batch_idx", batch_idx)
-        self.log("train_acc", acc, on_step=False, on_epoch=True)
-        self.log("train_loss", loss)
+        # Logs the fl-related metrics if fl_hparams is not None
+        if self.fl_hparams is not None:
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_train_loss",
+                loss,
+                on_step=False,
+                on_epoch=True,
+            )
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_train_acc",
+                acc,
+                on_step=False,
+                on_epoch=True,
+            )
+        else:
+            # Logs the accuracy per epoch (weighted average over batches)
+            self.log("batch_idx", batch_idx)
+            self.log("train_acc", acc, on_step=False, on_epoch=True)
+            self.log("train_loss", loss, on_step=False, on_epoch=True)
         return loss
 
     def validation_step(  # type: ignore
@@ -592,14 +646,33 @@ class CIFAR100(pl.LightningModule):
 
         Args:
             - batch (Tuple[Tensor, Tensor]): Batch of the validation data.
-            - batch_idx (int): Index of the given batch.
+            - batch_idx (int): Index of the batch.
         """
         imgs, labels = batch
-        preds: Tensor = self.model(imgs).argmax(dim=-1)
+        preds: Tensor = self.model(imgs)
+        loss: Tensor = self.loss_module(preds, labels)
+        preds = self.model(imgs).argmax(dim=-1)
         acc: Tensor = (labels == preds).float().mean()
-        # By default logs it per epoch (weighted average over batches)
-        self.log("batch_idx", batch_idx)
-        self.log("val_acc", acc)
+
+        # Logs the fl-related metrics if fl_hparams is not None
+        if self.fl_hparams is not None:
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_loss",
+                loss,
+                on_step=False,
+                on_epoch=True,
+            )
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_acc",
+                acc,
+                on_step=False,
+                on_epoch=True,
+            )
+        else:
+            # By default logs it per epoch (weighted average over batches), and returns it afterwards
+            self.log("batch_idx", batch_idx)
+            self.log("test_loss", loss, on_step=False, on_epoch=True)
+            self.log("test_acc", acc, on_step=False, on_epoch=True)
 
     def test_step(  # type: ignore
         self, batch: Tuple[Tensor, Tensor], batch_idx: int
@@ -608,11 +681,30 @@ class CIFAR100(pl.LightningModule):
 
         Args:
             - batch (Tuple[Tensor, Tensor]): Batch of the testing data.
-            - batch_idx (int): Index of the given batch.
+            - batch_idx (int): Index of the batch.
         """
         imgs, labels = batch
-        preds: Tensor = self.model(imgs).argmax(dim=-1)
+        preds: Tensor = self.model(imgs)
+        loss: Tensor = self.loss_module(preds, labels)
+        preds = self.model(imgs).argmax(dim=-1)
         acc: Tensor = (labels == preds).float().mean()
-        # By default logs it per epoch (weighted average over batches), and returns it afterwards
-        self.log("batch_idx", batch_idx)
-        self.log("test_acc", acc)
+
+        # Logs the fl-related metrics if fl_hparams is not None
+        if self.fl_hparams is not None:
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_loss",
+                loss,
+                on_step=False,
+                on_epoch=True,
+            )
+            self.log(
+                f"{self.fl_hparams.get('experiment_name', 'default')}_test_acc",
+                acc,
+                on_step=False,
+                on_epoch=True,
+            )
+        else:
+            # By default logs it per epoch (weighted average over batches), and returns it afterwards
+            self.log("batch_idx", batch_idx)
+            self.log("test_loss", loss, on_step=False, on_epoch=True)
+            self.log("test_acc", acc, on_step=False, on_epoch=True)
